@@ -1,16 +1,21 @@
 #' Plot Method for Maps
 #'
-#' This function maps raster layer values.
+#' This function maps raster and point data.
 #' A key showing how the colors map to raster values is shown below the map.
 #' The width and height of the graphics region will be automagically determined in some cases.
 #'
-#' @param r RasterLayer, SpatialGridDataFrame, or CRS.
-#'   A raster layer with values to be plotted or a coordinate reference system (CRS).
+#' @param r Raster*, SpatialGridDataFrame, or CRS.
+#'   An object that can be converted to a raster layer, or a coordinate reference system (CRS).
+#' @param p SpatialPointsDataFrame.
+#'   Spatial point data to be plotted.
+#' @param ...
+#'   Graphics parameters to be passed to \code{\link{AddPoints}}.
+#'   Unused if \code{p = NULL}.
 #' @param layer integer.
-#'   Column to use in the SpatialGridDataFrame.
-#' @param att numeric or character.
-#'   Variable identifying the levels attribute to use in the Raster Attribute Table (RAT).
-#'   This argument requires \code{r} values that are of class factor.
+#'   Layer to extract from if \code{r} is of class RasterStack/Brick or SpatialGridDataFrame.
+#' @param att integer or character.
+#'   The levels attribute to use in the Raster Attribute Table (RAT);
+#'   requires \code{r} values of class factor.
 #' @param n integer.
 #'   Desired number of intervals to partition the range of raster values (or \code{zlim} if specified) (optional).
 #' @param breaks numeric.
@@ -23,18 +28,18 @@
 #'   Vector of length 2 giving the minimum and maximum raster values for which colors should be plotted.
 #' @param asp numeric.
 #'   The \emph{y/x} aspect ratio for spatial axes.
+#'   Defaults to 1 (one unit on the \emph{x}-axis equals one unit on the \emph{y}-axis) when \code{r} is projected,
+#'   otherwise, a calculated value based on axes limits is used.
 #' @param extend.xy logical.
 #'   If true, the spatial limits will be extended to the next tick mark on the axes beyond the grid extent.
 #' @param extend.z logical.
 #'   If true, the raster value limits will be extended to the next tick mark on the color key beyond the measured range.
 #' @param reg.axs logical.
 #'   If true, the spatial data range is extended.
-#' @param trim.r logical.
-#'   If true, the outer rows and columns that consist of all NA values will be removed.
 #' @param dms.tick logical.
-#'   If true, the axes tickmarks are specified in degrees, minutes, and decimal seconds.
+#'   If true and \code{r} is projected, the axes tickmarks are specified in degrees, minutes, and decimal seconds (DMS).
 #' @param bg.lines logical.
-#'   If true, the graticule is drawn in back of the raster layer using white lines and a grey background.
+#'   If true, grids or graticules are drawn in back of the raster layer using white lines and a grey background.
 #' @param bg.image RasterLayer.
 #'   An image to drawn in back of the main raster layer \code{r}.
 #' @param bg.image.alpha numeric.
@@ -64,8 +69,7 @@
 #' @param explanation character.
 #'   Label explaining the raster cell value.
 #' @param credit character.
-#'   Label crediting the base map;
-#'   by default, the character string describing the raster layer's (\code{r}) projection and datum in the PROJ.4 format.
+#'   Label crediting the base map.
 #' @param shade list.
 #'   If specified, a semi-transparent shade layer is drawn on top of the raster layer.
 #'   This layer is described using a list of arguments supplied to \code{raster::hillShade} function.
@@ -98,10 +102,11 @@
 #'   Specifying this argument will start a graphics device driver for producing a
 #'   PDF or PNG file format---the file extension determines the format type.
 #'   The width and height of the graphics region will be automagically determined and
-#'   included with the function's returned values, see "Value" section for details.
-#'   These graphical parameters can be used when creating similar map layouts in dynamic reports
-#'   or whenever further elements will be added to an existing plot generated by this function
-#'   (possible when \code{file = NULL}).
+#'   included with the function's returned values, see "Value" section for details;
+#'   these device dimensions can be useful when creating similar map layouts in dynamic reports.
+#' @param close.file logical.
+#'   If true, the graphics device driver is shut down when the function exits.
+#'   Unused if \code{file = NULL}
 #' @param useRaster logical.
 #'   If true, a bitmap raster is used to plot \code{r} instead of using polygons.
 #'   If \code{UseRaster} is not specified, raster images are used when the \code{getOption("preferRaster")} is true.
@@ -127,7 +132,7 @@
 #' @export
 #'
 #' @examples
-#' r <- raster::raster(nrow = 10, ncol = 10)
+#' r <- raster::raster(nrow = 10, ncol = 10, crs = NA)
 #' r[] <- 1L
 #' r[51:100] <- 2L
 #' r[3:6, 1:5] <- 8L
@@ -140,22 +145,21 @@
 #' PlotMap(r, att = "code")
 #'
 #' r <- raster::raster(system.file("external/test.grd", package="raster"))
-#' credit <- "Label crediting the base map."
+#' credit <- "Label crediting the map."
 #' explanation <- "Label explaining the raster cell value."
-#' PlotMap(r, scale.loc = "topleft", dms.tick = TRUE, trim.r = TRUE,
-#'         credit = credit, explanation = explanation)
+#' PlotMap(r, scale.loc = "bottomright", dms.tick = TRUE, credit = credit,
+#'         explanation = explanation)
 #' data(meuse, package = "sp")
 #' sp::coordinates(meuse) = ~ x + y
 #' points(meuse)
 #'
-#' val <- PlotMap(r, scale.loc = "topleft", dms.tick = TRUE, trim.r = TRUE,
-#'                credit = credit, explanation = explanation,
-#'                file = "Rplots1.pdf")
+#' val <- PlotMap(r, scale.loc = "topleft", dms.tick = TRUE, credit = credit,
+#'                explanation = explanation, file = "Rplots1.pdf")
 #' print(val)
 #'
 #' pdf(file = "Rplots2.pdf", width = val$din[1], height = val$din[2])
-#' PlotMap(r, scale.loc = "topleft", dms.tick = TRUE, trim.r = TRUE,
-#'         credit = credit, explanation = explanation)
+#' PlotMap(r, scale.loc = "topleft", dms.tick = TRUE, credit = credit,
+#'         explanation = explanation)
 #' points(meuse)
 #' dev.off()
 #'
@@ -163,15 +167,18 @@
 #' graphics.off()
 #'
 
-PlotMap <- function(r, layer=1, att=NULL, n, breaks, xlim=NULL, ylim=NULL,
-                    zlim=NULL, asp=1, extend.xy=FALSE, extend.z=FALSE,
-                    reg.axs=TRUE, trim.r=TRUE, dms.tick=FALSE, bg.lines=FALSE,
-                    bg.image=NULL, bg.image.alpha=1, pal=NULL, col=NULL,
-                    max.dev.dim=c(43, 56), labels=NULL, scale.loc="bottomleft",
-                    arrow.loc=NULL, explanation=NULL, credit=proj4string(r),
-                    shade=NULL, contour.lines=NULL, rivers=NULL, lakes=NULL,
-                    roads=NULL, draw.key=NULL, draw.raster=TRUE, file=NULL,
-                    useRaster) {
+PlotMap <- function(r, p=NULL, ..., layer=1, att=NULL, n=NULL, breaks=NULL,
+                    xlim=NULL, ylim=NULL, zlim=NULL, asp=NULL,
+                    extend.xy=FALSE, extend.z=FALSE,
+                    reg.axs=TRUE, dms.tick=FALSE, bg.lines=FALSE, bg.image=NULL,
+                    bg.image.alpha=1, pal=NULL, col=NULL, max.dev.dim=c(43, 56),
+                    labels=NULL, scale.loc=NULL, arrow.loc=NULL, explanation=NULL,
+                    credit=NULL, shade=NULL, contour.lines=NULL,
+                    rivers=NULL, lakes=NULL, roads=NULL, draw.key=NULL,
+                    draw.raster=TRUE, file=NULL, close.file=TRUE, useRaster) {
+
+  if (!is.null(p) && !inherits(p, "SpatialPoints"))
+    stop("spatial point data is the incorrect class")
 
   if (!is.null(bg.image) && !inherits(bg.image, "RasterLayer"))
     stop("background image is the incorrect class")
@@ -186,15 +193,20 @@ PlotMap <- function(r, layer=1, att=NULL, n, breaks, xlim=NULL, ylim=NULL,
               is.numeric(ylim) && length(ylim) == 2 && all(!is.na(ylim))
     if (!is.lim && is.null(bg.image))
       stop("spatial limits must be specified")
-    e <- raster::extent(if (is.lim) c(xlim, ylim) else bg.image)
+    e <- extent(if (is.lim) c(xlim, ylim) else bg.image)
     r <- raster(e, crs=r)
     r[] <- NA
   }
 
-  if (inherits(r, "SpatialGridDataFrame"))
+  if (!is.null(p)) try(p <- spTransform(p, r@crs), silent=TRUE)
+
+  if (is.null(asp) && !is.na(rgdal::CRSargs(raster::crs(r)))) asp <- 1
+
+  is.dms <- dms.tick && !is.na(CRSargs(r@crs))
+
+  if (inherits(r, c("RasterStack", "RasterBrick", "SpatialGrid")))
     r <- raster(r, layer=layer)
-  if (!inherits(r, "RasterLayer"))
-    stop("raster layer is the incorrect class")
+  if (!inherits(r, "RasterLayer")) stop("raster layer is the incorrect class")
 
   if (raster::is.factor(r)) {
     att.tbl <- raster::levels(r)[[1]]
@@ -212,87 +224,41 @@ PlotMap <- function(r, layer=1, att=NULL, n, breaks, xlim=NULL, ylim=NULL,
     r <- rr
   }
 
-  if (is.null(xlim))
-    xlim <- c(NA, NA)
-  if (is.null(ylim))
-    ylim <- c(NA, NA)
-  if (is.null(zlim))
-    zlim <- c(NA, NA)
+  xl <- if (is.null(xlim)) c(NA, NA) else xlim
+  yl <- if (is.null(ylim)) c(NA, NA) else ylim
+  zl <- if (is.null(zlim)) c(NA, NA) else zlim
 
-  e <- as.vector(extent(r))
-  if (!is.na(xlim[1]))
-    e[1] <- xlim[1]
-  if (!is.na(xlim[2]))
-    e[2] <- xlim[2]
-  if (!is.na(ylim[1]))
-    e[3] <- ylim[1]
-  if (!is.na(ylim[2]))
-    e[4] <- ylim[2]
-  r <- crop(r, extent(e))
-
-  if (trim.r && !all(is.na(r[])))
-    r <- trim(r)
-
-  xran <- bbox(r)[1, ]
-  yran <- bbox(r)[2, ]
-
-  if (extend.xy) {
-    default.xlim <- range(pretty(xran))
-    default.ylim <- range(pretty(yran))
-  } else {
-    if (reg.axs) {
-      buf <- diff(xran) * 0.04
-      default.xlim <- c(xran[1] - buf, xran[2] + buf)
-      default.ylim <- c(yran[1] - (buf * asp), yran[2] + (buf * asp))
-    } else {
-      default.xlim <- range(xran)
-      default.ylim <- range(yran)
-    }
-  }
-
-  if (is.na(xlim[1]))
-    xlim[1] <- default.xlim[1]
-  if (is.na(xlim[2]))
-    xlim[2] <- default.xlim[2]
-  if (is.na(ylim[1]))
-    ylim[1] <- default.ylim[1]
-  if (is.na(ylim[2]))
-    ylim[2] <- default.ylim[2]
+  e <- cbind(as.vector(extent(r)), if (is.null(p)) NULL else as.vector(extent(p)))
+  e <- c(min(e[1, ]), max(e[2, ]), min(e[3, ]), max(e[4, ]))
+  if (!is.na(xl[1])) e[1] <- xl[1]
+  if (!is.na(xl[2])) e[2] <- xl[2]
+  if (!is.na(yl[1])) e[3] <- yl[1]
+  if (!is.na(yl[2])) e[4] <- yl[2]
+  r <- crop(r, extent(e), snap="near")
+  if (!is.null(p)) p <- crop(p, extent(e), snap="near")
 
   zran <- range(r[], finite=TRUE)
   if (anyNA(zran)) {
     n <- 0
   } else {
-    default.zlim <- range(if (extend.z) pretty(zran) else zran)
+    default.zl <- if (extend.z) range(pretty(zran, n=6)) else zran
     if (raster::is.factor(r)) {
       at1 <- raster::levels(r)[[1]][, "ID"]
       breaks <- c(0.5, at1 + 0.5)
-      zlim <- range(breaks)
+      zl <- range(breaks, finite=TRUE)
     } else {
-      if (all(is.na(zlim))) {
-        if (missing(breaks)) {
-          if (missing(n))
-            zlim <- if (extend.z) range(pretty(zran)) else zran
-        } else {
-          zlim <- range(breaks)
-        }
+      if (all(is.na(zl)) && !is.null(breaks)) {
+        zl <- range(breaks, finite=TRUE)
       } else {
-        if (is.na(zlim[1]))
-          zlim[1] <- default.zlim[1]
-        if (is.na(zlim[2]))
-          zlim[2] <- default.zlim[2]
+        if (is.na(zl[1])) zl[1] <- default.zl[1]
+        if (is.na(zl[2])) zl[2] <- default.zl[2]
       }
-      if (missing(breaks)) {
-        if (missing(n) || n > 200L) {
-          breaks <- seq(zlim[1], zlim[2], length.out=200L)
-          at1 <- if (extend.z) pretty(zran) else pretty(zlim)
+      if (is.null(breaks)) {
+        if (is.null(n) || n > 200L) {
+          breaks <- seq(zl[1], zl[2], length.out=200L)
+          at1 <- pretty(if (extend.z) zran else zl, n=6)
         } else {
-          if (all(is.na(zlim))) {
-            breaks <- pretty(zran, n=n)
-            zlim <- if (extend.z) range(breaks) else zran
-          } else {
-            breaks <- pretty(zlim, n=n)
-          }
+          breaks <- pretty(zl, n=n)
           at1 <- breaks
         }
       } else {
@@ -300,22 +266,147 @@ PlotMap <- function(r, layer=1, att=NULL, n, breaks, xlim=NULL, ylim=NULL,
       }
     }
     n <- length(breaks) - 1L
+    r[r[] < zl[1] | r[] > zl[2]] <- NA
   }
 
-  if (!is.logical(draw.key)) draw.key <- if (n == 0) FALSE else TRUE
+  if (!all(is.na(r[]))) r <- trim(r)
 
-  if (dms.tick) {
+  xran <- range(c(bbox(r)[1, ]), if (is.null(p)) NULL else bbox(p)[1, ])
+  yran <- range(c(bbox(r)[2, ]), if (is.null(p)) NULL else bbox(p)[2, ])
+
+  if (extend.xy) {
+    default.xl <- range(pretty(xran))
+    default.yl <- range(pretty(yran))
+  } else {
+    if (reg.axs) {
+      buf <- diff(xran) * 0.04
+      aspect <- ifelse(is.null(asp), (diff(xran) / diff(yran)), asp)
+      default.xl <- c(xran[1] - buf, xran[2] + buf)
+      default.yl <- c(yran[1] - (buf * aspect), yran[2] + (buf * aspect))
+    } else {
+      default.xl <- range(xran)
+      default.yl <- range(yran)
+    }
+  }
+
+  if (is.na(xl[1])) xl[1] <- default.xl[1]
+  if (is.na(xl[2])) xl[2] <- default.xl[2]
+  if (is.na(yl[1])) yl[1] <- default.yl[1]
+  if (is.na(yl[2])) yl[2] <- default.yl[2]
+
+  if (is.null(draw.key)) draw.key <- ifelse(n == 0, FALSE, TRUE)
+
+  csi <- 0.2  # assumed line height in inches
+  mai2 <- c(2, 3, 2, 2) * csi
+  if (draw.key) {
+    y1 <- csi
+    FUN <- function(i) {if (is.null(i)) 0 else length(strsplit(i, "\n")[[1]])}
+    mai2[1] <- (FUN(credit) + 1) * (csi * 0.6)
+    mai1 <- c(1.2, 3, NA, 2) * csi
+    mai1[3] <- FUN(explanation) * (csi * 0.7)
+  } else {
+    y1 <- 0
+    mai1 <- c(0, 0, 0, 0)
+  }
+
+  inches.in.pica <- 1 / 6
+  max.dev.dim <- max.dev.dim * inches.in.pica
+
+  if (is.null(file)) {
+    if (grDevices::dev.cur() == 1) grDevices::dev.new()
+    dev.dim <- grDevices::dev.size()
+    w <- dev.dim[1]
+    h1 <- y1 + mai1[1] + mai1[3]
+    h2 <- dev.dim[2] - h1
+    h <- h1 + h2
+  } else {
+    w <- max.dev.dim[1]
+    aspect <- ifelse(is.null(asp), (diff(xl) / diff(yl)), asp)
+    repeat {
+      y2 <- (w - mai2[2] - mai2[4]) * (diff(yl) / diff(xl)) * aspect
+      h2 <- y2 + mai2[1] + mai2[3]
+      h1 <- y1 + mai1[1] + mai1[3]
+      h <- h1 + h2
+      if (h > max.dev.dim[2]) w <- w - 0.01 else break
+    }
+    ext <- tolower(tools::file_ext(file))
+    if (ext == "pdf") {
+      grDevices::pdf(file, width=w, height=h)
+    } else if (ext == "png") {
+      grDevices::png(file, width=w * 100, height=h * 100, res=100)
+    } else {
+      stop("file argument does not have a valid extension")
+    }
+    if (close.file) on.exit(grDevices::dev.off())
+  }
+
+  if (draw.key) {
+    cm.in.inches <- 2.54
+    heights <- c(h2/ h, graphics::lcm(h1 * cm.in.inches))
+    graphics::layout(matrix(c(2, 1), nrow=2, ncol=1), heights=heights)
+  } else {
+    graphics::layout(matrix(1, nrow=1, ncol=1))
+  }
+
+  lwd <- 0.5
+  cex <- 0.7
+  tcl <- 0.1 / graphics::par("csi")  # length for major ticks is 0.1 inches
+
+  if (is.character(col)) {
+    if (length(col) != n) stop("number of specified colors is incorrect")
+    cols <- col
+  } else {
+    if (is.function(pal)) {
+      cols <- pal(n)
+    } else {
+      if (requireNamespace("colorspace", quietly=TRUE))
+        cols <- colorspace::rainbow_hcl(n, start=0.0, end=(360 * (n - 1) / n) * 0.8)
+      else
+        cols <- grDevices::rainbow(n, start=0.0, end=0.8)
+    }
+  }
+  if (!all(.AreColors(cols))) stop("colors are not valid")
+
+  # plot color key
+  if (draw.key & n > 0) {
+    is.categorical <- raster::is.factor(r)
+    at <- if (is.null(labels$at)) at1 else labels$at
+    if (is.null(labels$labels))
+      labels <- if (is.categorical) raster::levels(r)[[1]][, "att"] else TRUE
+    else
+      labels <- labels$labels
+    AddColorKey(mai=mai1, is.categorical=raster::is.factor(r),
+                breaks=breaks, col=cols, at=at, labels=labels,
+                explanation=explanation)
+  } else if (draw.key) {
+    op <- graphics::par(mai=c(0, 0, 0, 0))
+    graphics::plot.new()
+    graphics::par(op)
+  }
+
+  # plot map
+  graphics::par(mai=mai2)
+  plot(NA, type="n", xlim=xl, ylim=yl, xaxs="i", yaxs="i", bty="n",
+       xaxt="n", yaxt="n", xlab="", ylab="", asp=asp)
+  usr <- graphics::par("usr")
+
+  if (is.null(file)) {
+    xl <- usr[1:2]
+    yl <- usr[3:4]
+  }
+
+  if (is.dms) {
     al <- list()
-    al[[1]] <- Lines(list(Line(rbind(c(xlim[1], ylim[1]),
-                                     c(xlim[2], ylim[1])))), ID="al1")
-    al[[2]] <- Lines(list(Line(rbind(c(xlim[1], ylim[1]),
-                                     c(xlim[1], ylim[2])))), ID="al2")
-    al[[3]] <- Lines(list(Line(rbind(c(xlim[1], ylim[2]),
-                                     c(xlim[2], ylim[2])))), ID="al3")
-    al[[4]] <- Lines(list(Line(rbind(c(xlim[2], ylim[1]),
-                                     c(xlim[2], ylim[2])))), ID="al4")
+    al[[1]] <- Lines(list(Line(rbind(c(xl[1], yl[1]),
+                                     c(xl[2], yl[1])))), ID="al1")
+    al[[2]] <- Lines(list(Line(rbind(c(xl[1], yl[1]),
+                                     c(xl[1], yl[2])))), ID="al2")
+    al[[3]] <- Lines(list(Line(rbind(c(xl[1], yl[2]),
+                                     c(xl[2], yl[2])))), ID="al3")
+    al[[4]] <- Lines(list(Line(rbind(c(xl[2], yl[1]),
+                                     c(xl[2], yl[2])))), ID="al4")
     sl <- SpatialLines(al, proj4string=r@crs)
-    sl.dd <- spTransform(sl, CRS("+proj=longlat +datum=WGS84"))
+    sl.dd <- spTransform(sl, CRS("+init=epsg:4326"))
     e.dd <- pretty(range(bbox(sl.dd)[1, ]))
     n.dd <- pretty(range(bbox(sl.dd)[2, ]))
     grd.dd <- gridlines(sl.dd, easts=e.dd, norths=n.dd, ndiscr=1000)
@@ -332,117 +423,23 @@ PlotMap <- function(r, layer=1, att=NULL, n, breaks, xlim=NULL, ylim=NULL,
     at2[[3]] <- as.vector(coordinates(pts[ids == "al3 NS", ])[, 1])
     at2[[4]] <- as.vector(coordinates(pts[ids == "al4 EW", ])[, 2])
 
-    xlabs <- .FormatDMS(dd2dms(coordinates(pts.dd[ids == "al3 NS", ])[, 1],
-                               NS=FALSE))
-    ylabs <- .FormatDMS(dd2dms(coordinates(pts.dd[ids == "al2 EW", ])[, 2],
-                               NS=TRUE))
+    xlabs <- .FormatDMS(dd2dms(coordinates(pts.dd[ids == "al3 NS", ])[, 1], NS=FALSE))
+    ylabs <- .FormatDMS(dd2dms(coordinates(pts.dd[ids == "al2 EW", ])[, 2], NS=TRUE))
   } else {
     at2 <- list()
-    at2[[1]] <- pretty(xlim)
-    at2[[2]] <- pretty(ylim)
+    at2[[1]] <- pretty(xl)
+    at2[[2]] <- pretty(yl)
     at2[[3]] <- at2[[1]]
     at2[[4]] <- at2[[2]]
-    xlabs <- prettyNum(at2[[1]])
-    ylabs <- prettyNum(at2[[2]])
-    if (extend.xy)
-     ylabs[length(ylabs)] <- ""
+    xlabs <- prettyNum(at2[[1]], big.mark=",")
+    ylabs <- prettyNum(at2[[2]], big.mark=",")
+    if (extend.xy) ylabs[length(ylabs)] <- ""
   }
 
-  inches.in.pica <- 1 / 6
-
-  mar2 <- c(1, 3, 2, 2)
-  if (is.character(credit)) mar2[1] <- 2
-  if (draw.key) {
-    y1 <- 1
-    mar1 <- c(2, 3, 1, 2)
-  } else {
-    y1 <- 0
-    mar1 <- c(0, 0, 0, 0)
-  }
-
-  if (is.null(file)) {
-    if (grDevices::dev.cur() == 1) grDevices::dev.new()
-    dev.dim <- grDevices::dev.size() / inches.in.pica
-    w <- dev.dim[1]
-    h1 <- y1 + mar1[1] + mar1[3]
-    h2 <- dev.dim[2] - h1
-    h <- h1 + h2
-  } else {
-    w <- max.dev.dim[1]
-    repeat {
-      y2 <- (w - mar2[2] - mar2[4]) * (diff(ylim) / diff(xlim)) * asp
-      h2 <- y2 + mar2[1] + mar2[3]
-      h1 <- y1 + mar1[1] + mar1[3]
-      h <- h1 + h2
-      if (h > max.dev.dim[2]) w <- w - 0.01 else break
-    }
-    wi <- w * inches.in.pica
-    hi <- h * inches.in.pica
-    ext <- tolower(tools::file_ext(file))
-    if (ext == "pdf") {
-      grDevices::pdf(file, width=wi, height=hi)
-    } else if (ext == "png") {
-      grDevices::png(file, width=wi * 100, height=hi * 100, res=100)
-    } else {
-      stop("file argument does not have a valid extension")
-    }
-    on.exit(grDevices::dev.off())
-  }
-
-  if (draw.key)
-    graphics::layout(matrix(c(2, 1), nrow=2, ncol=1), heights=c(h2, h1) / h)
-  else
-    graphics::layout(matrix(1, nrow=1, ncol=1))
-
-  lwd <- 0.5  # line weight
-  cex <- 0.7  # magnification of text relative to the default
-  tcl <- 7.2 / graphics::par("cra")[2]  # length of ticks, fraction of text height
-
-  if (is.character(col)) {
-    if (length(col) != n) stop("number of specified colors is incorrect")
-    cols <- col
-  } else {
-    if (is.function(pal)) {
-      cols <- pal(n)
-    } else {
-      if (requireNamespace("colorspace", quietly=TRUE))
-        cols <- colorspace::rainbow_hcl(n, start=0.0, end=(360 * (n - 1) / n) * 0.8)
-      else
-        cols <- grDevices::rainbow(n, start=0.0, end=0.8)
-    }
-  }
-  if (!all(.AreColors(cols)))
-    stop("colors are not valid")
-
-  # Plot color key
-
-  if (draw.key & n > 0) {
-    is.categorical <- raster::is.factor(r)
-    at <- if (is.null(labels$at)) at1 else labels$at
-    if (is.null(labels$labels))
-      labels <- if (is.categorical) raster::levels(r)[[1]][, "att"] else TRUE
-    else
-      labels <- labels$labels
-    AddColorKey(mai=mar1 * inches.in.pica, is.categorical=raster::is.factor(r),
-                breaks=breaks, col=cols, at=at, labels=labels,
-                explanation=explanation)
-  } else if (draw.key) {
-    op <- graphics::par(mar=c(0, 0, 0, 0))
-    graphics::plot.new()
-    graphics::par(op)
-  }
-
-  # Plot map
-
-  graphics::par(mai=mar2 * inches.in.pica)
-  plot(NA, type="n", xlim=xlim, ylim=ylim, xaxs="i", yaxs="i", bty="n",
-       xaxt="n", yaxt="n", xlab="", ylab="", asp=asp)
-
   if (bg.lines) {
-    usr <- graphics::par()$usr
     graphics::rect(xleft=usr[1], ybottom=usr[3], xright=usr[2], ytop=usr[4],
                    col="#E7E7E7", border=NA)
-    if (dms.tick) {
+    if (is.dms) {
       plot(spTransform(grd.dd, r@crs), lwd=lwd, col="#FFFFFF", add=TRUE)
     } else {
       graphics::abline(v=at2[[1]], lwd=lwd, col="#FFFFFF")
@@ -460,7 +457,7 @@ PlotMap <- function(r, layer=1, att=NULL, n, breaks, xlim=NULL, ylim=NULL,
   }
 
   if (draw.raster & n > 0) {
-    raster::image(r, maxpixels=length(r), useRaster=useRaster, zlim=zlim,
+    raster::image(r, maxpixels=length(r), useRaster=useRaster, zlim=zl,
                   col=cols, add=TRUE, breaks=breaks)
     if (is.list(shade)) {
       zfact <- as.numeric(shade[["z.factor"]])
@@ -524,30 +521,35 @@ PlotMap <- function(r, layer=1, att=NULL, n, breaks, xlim=NULL, ylim=NULL,
     color <- ifelse(length(color) == 1 && !is.na(color), color, "#1F1F1F")
     drawl <- ifelse(length(drawl) == 1 && !is.na(drawl), drawl, TRUE)
     metho <- ifelse(length(metho) == 1 && !is.na(metho), metho, "flattest")
-    contour.breaks <- if (n + 1L > 20L) pretty(zlim, 20L) else breaks
+    contour.breaks <- if (n + 1L > 20L) pretty(zl, 20L) else breaks
     ncontours <- length(contour.breaks)
     raster::contour(r, maxpixels=length(r), levels=contour.breaks,
-                    labels=formatC(contour.breaks, big.mark=","), xlim=xlim,
-                    ylim=ylim, zlim=zlim, labcex=0.5, drawlabels=drawl,
+                    labels=formatC(contour.breaks, big.mark=","),
+                    xlim=xl, ylim=yl, zlim=zl, labcex=0.5, drawlabels=drawl,
                     method=metho, axes=FALSE, col=color, lwd=lwd, add=TRUE)
   }
 
+  if (!is.null(p)) AddPoints(p, xlim=xlim, ylim=ylim, zlim=zlim, ...)
+
   graphics::axis(1, at=at2[[1]], labels=FALSE, lwd=-1, lwd.ticks=lwd, tcl=tcl,
                  cex.axis=cex)
-  if (dms.tick)
+
+  if (is.dms)
     graphics::axis(2, at=at2[[2]], labels=ylabs, lwd=-1, lwd.ticks=lwd, hadj=0,
-                   tcl=tcl, cex.axis=cex, las=1)
+                   tcl=tcl, cex.axis=cex, las=1, mgp=c(3, 1.1, 0))
   else
-    graphics::axis(2, at=at2[[2]], labels=ylabs, lwd=-1, lwd.ticks=lwd, padj=2,
-                   tcl=tcl, cex.axis=cex)
-  graphics::axis(3, at=at2[[3]], labels=xlabs, lwd=-1, lwd.ticks=lwd, padj=2,
-                 tcl=tcl, cex.axis=cex)
+    graphics::axis(2, at=at2[[2]], labels=ylabs, lwd=-1, lwd.ticks=lwd,
+                   tcl=tcl, cex.axis=cex, mgp=c(3, 0.1, 0))
+  graphics::axis(3, at=at2[[3]], labels=xlabs, lwd=-1, lwd.ticks=lwd,
+                 tcl=tcl, cex.axis=cex, mgp=c(3, 0.1, 0))
   graphics::axis(4, at=at2[[4]], labels=FALSE, lwd=-1, lwd.ticks=lwd, tcl=tcl,
                  cex.axis=cex)
   graphics::box(lwd=lwd)
 
-  if (is.character(credit))
-    graphics::mtext(credit, side=1, line=-0.5, padj=1, adj=0, cex=0.6, col="#C0C0C0")
+  if (is.character(credit)) {
+    line <- -1 * graphics::strheight("M", "inches", cex=1) / csi
+    graphics::mtext(credit, side=1, line=line, padj=1, adj=0, cex=0.6, col="#C0C0C0")
+  }
 
   if (!is.null(scale.loc)) {
     txt <- strsplit(proj4string(r), " ")[[1]]
@@ -556,17 +558,15 @@ PlotMap <- function(r, layer=1, att=NULL, n, breaks, xlim=NULL, ylim=NULL,
     AddScaleBar(asp, unit, lonlat, scale.loc)
   }
 
-  if (!is.null(arrow.loc))
-    .AddNorthArrow(arrow.loc, r@crs, cex)
+  if (!is.null(arrow.loc)) .AddNorthArrow(arrow.loc, r@crs, cex)
 
-  invisible(list(din=graphics::par("din"),
-                 usr=graphics::par("usr"),
-                 heights=c(h2, h1) / h))
+  invisible(list(din=graphics::par("din"), usr=usr, heights=c(h2, h1) / h))
 }
 
 
 .LocateMidTicks <- function(side) {
-  ran <- if (side %in% c(1, 3)) graphics::par("usr")[1:2] else graphics::par("usr")[3:4]
+  usr <- graphics::par("usr")
+  ran <- if (side %in% c(1, 3)) usr[1:2] else usr[3:4]
   at <- graphics::axTicks(side)
   inc <- diff(range(at)) / (length(at) - 1)
   at.mid <- seq(min(at) - inc, max(at) + inc, by=inc / 2)
@@ -576,24 +576,18 @@ PlotMap <- function(r, layer=1, att=NULL, n, breaks, xlim=NULL, ylim=NULL,
 
 
 .AddNorthArrow <- function(loc, crs, cex) {
+  crs.dd <- CRS("+init=epsg:4326")
   usr <- graphics::par("usr")
-
   x.mid <- (usr[2] + usr[1]) / 2
   y.mid <- (usr[4] + usr[3]) / 2
-
-  crs.dd <- CRS("+proj=longlat +datum=WGS84")
-
   d <- 0.05 * (usr[4] - usr[3])
-
   xy <- rbind(c(x.mid, y.mid), c(x.mid, y.mid + d))
   sp.dd <- spTransform(SpatialPoints(xy, proj4string=crs), crs.dd)
   dd <- sp.dd@coords
-
   d.dd <- sqrt((dd[2, 1] - dd[1, 1])^2 + (dd[2, 2] - dd[1, 2])^2)
   dd <- rbind(dd[1, ], c(dd[1, 1],  dd[1, 2] + d.dd))
   sp.xy <- spTransform(SpatialPoints(dd, proj4string=crs.dd), crs)
   xy <- sp.xy@coords
-
   padx <- 0.1 * (usr[2] - usr[1])
   pady <- 0.1 * (usr[4] - usr[3])
   if (loc %in% c("bottomleft", "topleft"))
@@ -606,7 +600,6 @@ PlotMap <- function(r, layer=1, att=NULL, n, breaks, xlim=NULL, ylim=NULL,
     y0 <- usr[4] - pady
   x1 <- xy[2, 1] + x0 - xy[1, 1]
   y1 <- xy[2, 2] + y0 - xy[1, 2]
-
   a <- atan((y1 - y0) / (x1 - x0)) * (180 / pi)
   if (a > 45 && a <= 135) {
     pos <- 3
@@ -617,7 +610,6 @@ PlotMap <- function(r, layer=1, att=NULL, n, breaks, xlim=NULL, ylim=NULL,
   } else {
     pos <- 4
   }
-
   graphics::arrows(x0, y0, x1, y1, length=0.1)
   text(x1, y1, labels="N", pos=pos, cex=cex)
 }
@@ -640,4 +632,3 @@ PlotMap <- function(r, layer=1, att=NULL, n, breaks, xlim=NULL, ylim=NULL,
   sapply(x, function(i) tryCatch(is.matrix(grDevices::col2rgb(i)),
                                  error=function(e) FALSE))
 }
-
